__version__ = "0.3.0"

from aliases.space import AliasSpace
from aliases.string import AliasAwareString
from aliases.dict import AliasAwareDict

__all__ = [AliasSpace, AliasAwareDict, AliasAwareString]
